-- Copyright (c) 2012 Takahiro Shinozaki
-- 
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
-- 
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
-- 
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.

--------------------------------------------------------------------------------
-- |
-- Module      :  Main
-- Copyright   :  Takahiro Shinozaki 2012
-- License     :  MIT
-- 
-- Maintainer  :  www.ailab.tj.chiba-u.jp/~shinot
-- Stability   :  experimental
-- Portability :  portable
-- 
-- husky main module (The main module for decoding is DcdComps)
--------------------------------------------------------------------------------

{-# LANGUAGE ForeignFunctionInterface #-}
{-# LANGUAGE BangPatterns #-}

-- Flexible Functional Speech Recognizer
-- Written by Takahiro Shinozaki
-- Chiba University 2011-2012
-- Tokyo Institute of Technology 2010-1011
-- Supported by JST Research Seeds Quest Program 2010.1-2010.12

-- Version 2.1 2012.9.18   Pipeline based design and implementation of the whole decoder

-- Version 2.0 2012.6.4    Made a WFST package using FFI for speed up & memory saving

-- Version 1.0 2012.1.27   Improved efficiency of file inputs

-- Version 0.0 2010.6.21   (Very slow but it has been confirmed to work with 5000 vocabulary, 
--                         bigram and 3000-tied-sate HMM)

module Main (main) where

-- import System
import qualified Data.ByteString.Lazy.Char8 as L8
import qualified Data.List as List
import qualified Data.Vector as V
import System.Environment (getArgs)
import WFST
import GMM
import MHUtils
import DcdComps

main = do
  args <- getArgs
  case args of 
    [config, wfstf, spdff, osymf, scpf] -> do {
      config <- readConfig config;
      wfst <- wfstFromFile wfstf;
      spdfs <- spdfsFromFile spdff;
      scp <- readScpFile scpf;
      osyms <- outSymFromFile osymf;
      mapM_ (decodeWfstMain config wfst spdfs osyms) scp
      }
    _ -> putStrLn "error: husky config wfstf spdff osymf scpf"


decodeWfstMain config wfst spdfs osyms feaf = do {
  print feaf;
  feas <- htkParmFromFile feaf; seq feas (print (List.length (feaVec feas)));
  mapM_ L8.putStrLn $ 
      List.map (osyms V.!) $ 
      filter (>0) $ 
      List.map outLbl $ 
      progressiveBacktrack (50) (finSt wfst) $ 
      decodeForward config wfst spdfs (feaVec feas)
  }

  
-- Read feature list file
readScpFile :: String -> IO [String]
readScpFile fname = do {buf <- L8.readFile fname; return (List.map L8.unpack (L8.words buf))}
