-- Copyright (c) 2012 Takahiro Shinozaki
-- 
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
-- 
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
-- 
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-- THE SOFTWARE.

--------------------------------------------------------------------------------
-- |
-- Module      :  MHUtils
-- Copyright   :  Takahiro Shinozaki 2012
-- License     :  MIT
-- 
-- Maintainer  :  www.ailab.tj.chiba-u.jp/~shinot
-- Stability   :  experimental
-- Portability :  portable
-- 
-- A module for some utility functions
--------------------------------------------------------------------------------

{-# LANGUAGE BangPatterns #-}
{-# LANGUAGE ScopedTypeVariables #-}

-- My haskell utility functions

module MHUtils (
  mBtoI,
  mBtoD,
  memoize
  ) where

-- import System
import qualified Data.ByteString.Lazy.Char8 as L8
import Data.ByteString.Lex.Double
import Foreign.C.Types (CDouble)

import Data.Hashable
import Data.HashTable.IO as H
import System.IO.Unsafe

-- Bytestring to numeric conversions
mBtoD :: L8.ByteString -> CDouble
mBtoD bs = case unsafeReadDouble $ L8.toStrict bs of
			Nothing -> 0
			Just (k, rest) -> realToFrac k

mBtoI :: L8.ByteString -> Int
mBtoI bs = case L8.readInt bs of
			Nothing -> 0
			Just (k, rest) -> k


memoize :: (Eq k, Hashable k) => (k -> a) -> k -> a
memoize f = unsafePerformIO $ do
  ht :: H.BasicHashTable k a <- H.new 
  return $ \x -> unsafePerformIO $ do
    mbres <- H.lookup ht x
    case mbres of
      Nothing ->
        let res = f x in do
          H.insert ht x res
          return res
      Just res ->
          return res